#
#
# program to read the info about each DALI device
# display the serial number, UPC code and firmware versions
#
#

import RPi.GPIO as GPIO
import time
import serial
import sys

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=.2
)

def SetBank(DTR1, DTR):
	global bus
	global index
	ser.write('%shC3%02X\n'%(bus,DTR1))
	x=ser.read(3)
	ser.write('%shA3%02X\n'%(bus,DTR))
	x=ser.read(3)
	index = DTR
	return


def serchannel(Bytes):
	xgc=ser.read(1)
	if xgc== '1' :
		xgc=ser.read(Bytes)
		return xgc
	else :
		xgc1=ser.read(Bytes-1)
		return xgc+xgc1
	return

def ReadByte(cmd, dadd):
	global bus
	ser.write('%sh%02X%02X\n'%(bus,dadd*2+1,cmd)) 
	x=ser.read(1)
	if(x == '1'):  # bus 1
		x=ser.read(1)
	if(x == '2'):  # bus 1
		x=ser.read(1)
	if(x == '3'):  # bus 1
		x=ser.read(1)
	while(x == 'H'):  # dump other traffic
		ser.read(5)
		x=ser.read(1)
	if(x == 'J'):
		byte = int(ser.read(2),16)
		crlf = ser.read(1)
		return byte
	if(x == 'X'):
		byte = ser.read(1)
		print('\nCollision %s at %d'%(byte,dadd))
		crlf = ser.read(1)
		return -1
	if(x != 'N'):
		z=ser.read(10)
		print(' sync %s cmd=%d rest %s'%(x,cmd,z))
		return -1
	crlf=ser.read(1) # got an N, clear the crlf
	return -1


def readData(seed,dadd,base) :
	global index

	data = ReadByte(0xC5,dadd)
	if(data == -1):
		x = ser.read(1) # slow response?
		if( x == ''): # nothing at all?
			return -1
		if(x == '1'):  # bus 1
			x=ser.read(1)
		if( x == 'J' ) : # delayed
			print('delayed J...'),
			data = ser.read(2)
			crlf = ser.read(1)
		if( data == -1):
			z = ser.read(10) # not delayed read - garbage, dump
			print(' got %s not J, overrun %s'%(x,z))
			devDTR = ReadByte(0x98,dadd) # check DTR
			if(devDTR != -1):  # no DTR to read
				if(devDTR != index): # missed the response
					ser.write('hA3%02X\n'%index) # reload DTR
					ncrlf = ser.read(2)
					print('device %d DTR read %d not %d '%(dadd, devDTR,index) ),  # missed response
				else : 
					print('device %d missed query'%dadd),       # missed the query
					ser.read(10)
				data = ReadByte(0xC5,dadd) # try again
	index = index + 1
	seed = seed*256  + data
	return seed


GPIO.setmode(GPIO.BCM)
GPIO.setup(5,GPIO.IN)
GPIO.setup(6,GPIO.IN)

print('\n      ATX LED Consultants Inc - Fixture Read UPC, Version, SN \n')

ser.read(100000)

ser.write('v\n')  # lets get version number first
x=ser.read()
if x == "" :
	print('  Serial Port Conflict  \n\n')
	exit()

x=ser.read(2)
print('Hat HW version: %s'%(x))
y=ser.read(2)
print('Hat FW version: %s'%(y))
y=ser.read(2)
HW=0

if(y != '\n'):
	HW=int(y,16)
	print('Hat HW type: %s'%(HW))
	ser.read(1)

fadd = 0
bus = ""
counts =0

ser.read(100000)

if(len(sys.argv) > 1) :
	fadd = int(sys.argv[1])

if len(sys.argv) > 2 :
	text = sys.argv[2]
	if(text == "1"):
		ser.write("q1\n") # set 80 mA
		ser.read(18)
		time.sleep(1)
	if(text == "3"):
		ser.write("q3\n") # set 180 mA
		ser.read(18)
		time.sleep(1)
else:
	print(' no change in current')
ser.write('q\n')
x = ser.read(2)
v = ser.read(5)
i1 = ser.read(3)
i2 = ser.read(4)
i3 = ser.read(3)
i3 = ser.read(1)
crlf = ser.read(1)

if( HW != 2) :
	print('\n DALI Bus voltage,%s  Currents %s %s %s'%(v,i1,i2,i3))	


while fadd < 64 :

	byte = ReadByte(160,fadd)
	if byte < 0 :
		level = ReadByte(171,fadd)
		if(level >= 0):
			print('\n*%s   Addr %2d Arc %3d                   has a Transceiver '%(bus,fadd,level)),
		fadd += 1
		if( HW == 2 and fadd == 64 and bus != '1' ) :
			fadd = 0
			bus = '1'
		continue

	counts += 1

	level = ReadByte(171,fadd)
	if(level >= 0):
		print('\n*%s   Addr %2d Arc %3d                          also has a Transceiver '%(bus,fadd,level)),


	print('\n%s %2d Addr %2d Arc %3d '%(bus, counts,fadd,byte)),

	# dtr1, 0 as 0:3 

	SetBank(0,3)

	SN = 0
	Cupc = 0
	UPC = 0

	while(Cupc < 6 and UPC != -1 ) :
		UPC = readData(UPC,fadd,16)
		Cupc = Cupc+1
	
	if( UPC != -1 ):

		if( UPC == 722512406476 ): 
			UPC = "AL-WS-DALI-8B"
		elif( UPC == 722512406087 ):
			UPC = "AL-DALI-IO16"
		elif ( UPC == 784099948190):
			UPC = "AL-WS-DR2F32k"
		elif ( UPC == 784099948268 ):
			UPC = "AL-WS-DR2F"
		elif ( UPC == 784099947797 ):
			UPC = "AL-WS-DR2G"
		elif ( UPC == 722512407183 ):
			UPC ="AL-WS-DR2"
		elif( UPC == 722512407282 ):
			UPC = "AL-WS-010v"
		elif( UPC == 784099948350 ):
			UPC = "AL-PWS=DR1"
		elif( UPC == 784099947780 ):
			UPC = "AL-WS-DR1-PIR"
		elif( UPC == 722512407367 ):
			UPC = "AL-DA-Relay8"
		elif( UPC == 722512407350 ):
			UPC = "AL-PSE-4D"
		elif ( UPC == 281474976710655 ):
			UPC = "None"
		elif ( UPC == 4328719365 ) :
			UPC = "None"
		else:
			UPC = '%lx'%(UPC)

		fw = readData(0,fadd,16)
		hw = readData(0,fadd,16)
		SN = readData(0,fadd,16)
		SN = readData(SN,fadd,16)
		SN = readData(SN,fadd,16)
		SN = readData(SN,fadd,16)
	else:
		UPC = "None"
	print('%13s'%UPC),

	print('FW: %3d'%(fw)),

	print('HW: %d'%(hw)),

	if(SN != 4294967295 and SN != 0 ):
		print('SN (Hex): %08X'%(SN)),

	if("AL-WS-DR" in UPC) :

		y = readData(0,fadd,16)
		if(y != 255):
			print('N-Mode: %d'%(y)), # 15 = N-Way

		if( ReadByte(166,fadd) == 1 and UPC != "AL-WS-DR2"): # device type
			SetBank(5,9)
			y = readData(0,fadd,16)
			if(y != 255):
				print('Drive Mode: %02X'%(y)), # LED Driver mode

	fadd +=1

	if(HW == 2 and fadd == 64 and bus != '1'):
		fadd = 0
		print('')
		bus = '1'

