#!/usr/bin/python2.7
#
# program to read the memory banks of a device
#
#

import time
import serial
import sys

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=1,
)

 

def readData(seed,device,base) :
	global index
	ser.write('h%02xC5\n'%device)  # read memory
	x=ser.read(1)
	while ( x == 'H'):  # other bus traffic
		ser.read(4)
		x = ser.read(1)
	if( x == 'N'):
		crlf=ser.read(1)
		index = index + 1
		return 0
	if( x != 'J'):
		crlf = ser.read(1)
		w = ser.read(1)  # look for delay
		if( w == 'J' ) : # delayed
			print('d'),
		retry = 0
		while( x != "J" and retry < 3):
			z = ser.read(10) # clear garbage
			print('%s%s%s%s'%(x,crlf,w,z)),
 			ser.write('h%02x98\n'%device) # check DTR
			ser.read(1)
			x = ser.read(2)
			DTR = int(x,16)
			ser.read(1)
			if(DTR == index + 1): # missed the response
				ser.write('hA3%02X\n'%index) # reload DTR 
				N_CR = ser.read(2) 
				print(' r '),
			elif(DTR == index) : # missed the query
				print(' q '),
			else :
				print('??'), # DTR really out of synce
			ser.write('h%02xC5\n'%device) # read again
			x = ser.read(1)
			retry += 1
			if(retry == 3 ):
				return 0

		data=ser.read(2)
		ser.read(1)
	else : # normal read
		data=ser.read(2)
		ser.read(1)

	index = index + 1
	seed = seed*256 + int(data,base)
	return seed

def lsbFirst(device,base) :
	lsb = readData(0,device,base)
	return readData(0,device,base)*256 + lsb

def SetBank(Bank, Addr):
	global index
	ser.write('hC3%02x\n'%Bank)
	time.sleep(.1)
	x=ser.read(2)
	ser.write('hA3%02x\n'%Addr)
	index = Addr
	time.sleep(.1)
	x=ser.read(2)
	return;

print('\n      ATX LED Consultants Inc - Read Model, Type, UPC, Version, SN \n')


ser.write('v\n')  # lets get version number first
x=ser.read()
if x == "" :
	print(' Serial Port Conflict \n\n')
	exit()

x=ser.read(2)
print('Hat HW version: %s'%(x)),
y=ser.read(2)
y=int(y,10)
print('FW version: %d'%(y)),
x=ser.read(2)
if x != "" :
	print('HW Type %s'%(x)),
ser.read(1)
print('')

Vnoload = 48209

if len(sys.argv) < 2 :
	text = input("input address to read : ")
else:
	text = sys.argv[1]

device = int(text)*2+1

# dtr1, 0 as 0:3 
SetBank(0,3) # set bank 0: 3

# read memory - upc msb
ser.reset_input_buffer()
UPC = readData(0,device,16)
UPC = readData(UPC,device,16)
UPC = readData(UPC,device,16)
UPC = readData(UPC,device,16)
UPC = readData(UPC,device,16)
UPC = readData(UPC,device,16)
if( UPC != 281474976710655 ) :
	print('UPC: %ld'%(UPC))
else:
	print(' No UPC')

fw = readData(0,device,16)
print('DR2  FW version: %02d'%(fw)),
hwv = readData(0,device,16)
print('HW version: %02d'%(hwv))
atxtype =0

SN = readData(0,device,16)
SN = readData(SN,device,16)
SN = readData(SN,device,16)
SN = readData(SN,device,16)
if(SN != 4294967295):
	print('Serial Number Hex: %08X'%(SN))
	atxtype =1


if( atxtype  > 0) :

	ser.write('h%02x03\n'%(device))  # bump 1 if on, kill fade
	time.sleep(1)
	ser.write('h%02x90\n'%device)  # read status
	x=ser.read(1)
	loopend = 10
	while(x == 'J'):
		data=ser.read(2)
		status = int(data,16)
		if(( status & 0xa0 ) == 0 ) :
			break
		loopend = loopend-1
		if(loopend == 2):
			ser.write('h%02x01\n'%(device-1))  # turn a light on
			crlf = ser.read(2)
		else:
			if( status & 0x20 ) : # no cal yet
				ser.write('h%02x01\n'%(device-1))  # trigger cal
				crlf = ser.read(2)
		crlf = ser.read(1)
		print(' Device Not Ready %x'%status)
		time.sleep(3)
		ser.write('h%02x90\n'%device)  # read status
		x=ser.read(1)
	if(x == 'H'):
		crlf = ser.read(4)
	crlf = ser.read(1)

	SetBank(4,3)
	hours = lsbFirst(device,16)  # 3,4
	print('Uptime %d hours'%hours),

	hours = lsbFirst(device,16)  # 5,6
	print('Ontime %d hours'%hours)

	hourslsb = readData(0,device,16)  # 7, 8
	hours = readData(0,device,16)*256 + hourslsb
	SetBank(4,12)
	hours = readData(0,device,16)*10 + hours*100
	print('Watthours since reset %d Wh'%hours)

	SetBank(4,10) 
	Peak = readData(0,device,16)
	SetBank(4,13)
	watts = readData(0,device,16)
	Vin = lsbFirst(device,16)
	Right = lsbFirst(device,16)
	Left = lsbFirst(device,16)
	MilliAmps = 0
	if(Vin):
		MilliAmps = ((Right + Left)*1000) / Vin
	print(' Peak =%2d W Now =%2d W Right=%5d mW Left=%5d mW '%(Peak,watts,Right,Left))
	if(Vin != 48000 ) :
		print(' Vin=%5d V @ %3d mA now \n'%(Vin,MilliAmps))
	else :
		print('Input Current %3d mA now \n'%(MilliAmps))

SetBank(5,0)

print('  MyShort   MyGroup       Min       Max')

xdl=0
xd32=0
bit16 = 0
style = 0
val1 = 0
val2 = 0
val3 = 0
while( index < 84) :

	xd = readData(0,device,16)
	if( bit16 == 0) :
		if(style == 0):
			print(' %2d = %03d'%(index-1,xd)),
		else:
			val0 = val1
			val1 = val2
			val2 = val3
			val3 = xd
	if index & 1 == 0 :
		if  bit16 == 1 :
			print('  %5d'%(xdl+xd*256)),
	xdl = xd
	xd32 |= xd<<(((index-1)&3)*8)
	if( index & 3 == 0) :
		if(bit16 == 2) :
			print('     %12d.%d %5d kWh '%(xd32/10,xd32%10,xd32/600000)),
		if(style == 0):
			print(' ')
		if(style == 1) :
			print(' 44 = Height                                 %2d      %2d'%(val0,val1))
			print('      Jumper                                 %2d      %2d'%(val2,val3))
		if(style == 2) :
			RBinary = format(val0,"#05b")
			LBinary = format(val2,"#05b")
			print(' 48 = Status 5=High Test 4= Open Test 3=LED OK 2=on 1=open 0=short')
			if(val1):
				print('      Fan OK'),
			else :
				print('      No Fan'),
			print('                           %7s %7s'%(RBinary,LBinary))

		style = 0
		xd32=0
		if( index == 4 ):
			print('  PowerOn, FadeRate,     R -MaxVolts- L')
			bit16=0
		if( index == 8 ):
			print('      CCT, LED Mode,     R -Swap CCT- L, Mode 0=A, 1=4x2, 2=F, 3=CCT, 4=8x, 5=PIR, 9=CCTR')
			bit16=0
		if( index == 12 ):
			print('     FAIL, NW  Mode,    Dctrl     Level, NW 1=Split, 2=Fan, 4=PIR, 7=Timer')
			bit16=0
		if( index == 16 ):
			print('  Nway   , Main PB ,  UPS Max,  SA2 ARC, Nway bits: 3=swt, 2=hwt, 1=follow, 0=PB')
		if( index == 20 ):
			print('  Fan Hold Fan On D  Fan Idle  Fan Max')
		if( index == 24 ):
			print('                                          Right    Left')
			print(' 24 = MP On values                     '),
			bit16 = 1
		if( index ==  28  ):
			print('      Dim 2    Split      Left')
			print('      Warm      SA       Slider     FW')
			bit16=0
		if( index == 32 ):
			print(' 32 = Low Dim On Value                 '),
			bit16 = 1
		if( index == 36 ):
			print(' 36 = idle off voltage                 '),
		if( index == 40 ):
			print(' 40 = MP top Values                    '),
		if( index == 44 ):
			style = 1
			bit16 = 0
		if( index == 48 ):
			style = 2
		if( index == 52 ):
			if( fw < 17 ):
				print(' 52 = crossover Linear PWM             '),
			else:
				print(' 52 = MP 30 mA                         '),
			bit16 = 1
		if( index == 56 ):
			print(' 56 = Led Voltage                      '),
		if( index == 60 ):
			print(' 60 = Wattminutes            '),
			bit16 = 2
		if( index == 64 ):
			bit16 = 1
			print(' 64 = PNP @ MP min                     '),
		if( index == 68 ):
			print(' 68 = mOhm / Height                    '),
		if( index == 72 ):
			print(' 72 = LED Voltage @ 660mA              '),
		if( index == 76 ):
			print('      CCT Fade, 2nd Fade,  R -Offset- L')
			bit16=0
		if( index == 80 ):
			print('      R -Phy Max- L       Min2      Max2')
