#!/usr/bin/python
#
# program to read the input voltage to a AL-WS-DR2
# Calculate the resistance of the wire
#

import RPi.GPIO as GPIO
import time
import serial
import sys

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=.2,
)

#GPIO.setmode(GPIO.BCM)
#GPIO.setup(5,GPIO.IN)
#GPIO.setup(6,GPIO.IN)
 

def SetBank(Bank,Location) :
	global index
	ser.write('hC3%02X\n'%Bank)
	x=ser.read(2)
	ser.write('hA3%02X\n'%Location)
	index = Location
	x=ser.read(2)
	return

def NoFade(device) :
	FadeReady=16
	while(FadeReady != 0):
		ser.write('h%02X90\n'%device)
		x=ser.read(1)
		if(x != 'N'):
			FadeReady = int(ser.read(2),16) & 0x90 
		ser.read(1)
		time.sleep(.5)
	time.sleep(1)
	return

def readData(seed,device,base) :
	global index
	ser.write('h%02xC5\n'%device) 
	x=ser.read(1)
	if( x != 'J'):
		y = ser.read(1)
		x = ser.read(1)
		if( x == 'J' ) : # delayed
			print('d'),
		while( x != "J"):
			z = ser.read(10) # not delayed read
			print('%s%s%s'%(y,x,z))
 			ser.write('h%02x98\n'%device) # check DTR
			ser.read(1)
			x = ser.read(2)
			DTR = int(x,16)
			ser.read(1)
			if(DTR == index + 1): # missed the response
				ser.write('hA3%02X\n'%index)
				ser.read(2)
				print(' r '),
			elif(DTR == index) : # missed the query
				print(' q '),
			else :
				print('??'),
			ser.write('h%02xC5\n'%device) # read again
			x = ser.read(1)

		data=ser.read(2)
		ser.read(1)
	else : # normal read
		data=ser.read(2)
		ser.read(1)

	index = index + 1
	seed = seed*256 + int(data,base)
	return seed

def lsbFirst(device,base) :
	lsb = readData(0,device,base)
	return readData(0,device,base)*256 + lsb


print('\n      ATX LED Consultants Inc - Read Topology \n')


ser.write('v\n')  # lets get version number first
x=ser.read()
if x == "" :
	print('  Serial Port Conflict  \n\n')
	exit()

x=ser.read(2)
print('Hat HW version: %s'%(x))
y=ser.read(2)
y=int(y,10)
print('Hat FW version: %d'%(y))
x=ser.read(2)
if x != "" :
	print('HW Type %s'%(x))
ser.read(1)

if len(sys.argv) < 2 :
	text = input("input address to read : ")
else:
	text = sys.argv[1]

x=ser.read(100)

device = int(text)*2+1

# dtr1, 0 as 0:3 

ser.write('hFE00\n') # off
ser.read(100)

SetBank(0,9)

fw = readData(0,device,16)
print('FW version: %02d'%(fw))
hwv = readData(0,device,16)
print('HW version: %02d'%(hwv))


if( hwv  > 2) :
	SetBank(4,3)
	hours = lsbFirst(device,16)
	print('Uptime %d hours'%hours)
	hours = lsbFirst(device,16)
	print('Ontime %d hours'%hours)
	hourslsb = readData(0,device,16)
	hours = readData(0,device,16)*256 + hourslsb

	SetBank(4,12)
	hours = readData(0,device,16)/10 + hours*100
	print('Wh %d'%hours)
	if( hwv == 4):
		ser.write('h%02XA0\n'%device)
		ser.read(1)
		level = ser.read(2)
		ser.read(1)
		if(level != "00"):
			ser.write('h%02X00\n'%device) # off
			ser.read(2)
			NoFade(device)

		SetBank(4,14)
		Vnoload = lsbFirst(device,16)
		ser.write('h%02X05\n'%device) # on
		ser.read(2)
		NoFade(device)

		SetBank(4,13)
		watts = readData(0,device,16)
		Vin = lsbFirst(device,16)
		Right = lsbFirst(device,16)
		Left = lsbFirst(device,16)
		MilliAmps = ((Right + Left)*1000) / Vin
		Rin = 0
		if( MilliAmps > 100 ) :
			Rin = ((Vnoload-Vin)*1000)/ MilliAmps 
		print('\n watts %2d\n V noload %5d'%(watts,Vnoload))
		print(' Vin   %05d\n Right %05d mW\n Left  %05d mW\nTotal %05d mA\n %5d mOhm  %3d ft'%(Vin,Right,Left,MilliAmps,Rin, Rin/13))
		ser.write('h%02X00\n'%device) # off
		ser.read(2)


