#
#
# program to write to memory in a DALI driver
# see manual for more info ( AL-WS-DR2 etc)
#

import time
import serial
import sys

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=.1,
)

def readData(seed,address,base) :
	global index
	device = address*2 + 1

	ser.write('h%02xC5\n'%device)  # read memory
	x=ser.read(1)
	while ( x == 'H'):  # other bus traffic
		ser.read(4)
		x = ser.read(1)
	if( x == 'N'):
		crlf=ser.read(1)
		ser.write('h%02xC5\n'%device)  # read memory
		x=ser.read(1)
		print('retry read memory')
	if( x != 'J'):
		crlf = ser.read(1)
		w = ser.read(1)  # look for delay
		if( w == 'J' ) : # delayed
			print('d'),
		retry = 0
		while( x != "J" and retry < 3):
			z = ser.read(10) # clear garbage
			print('%s%s%s%s'%(x,crlf,w,z)),
			ser.write('h%02x98\n'%device) # check DTR
			ser.read(1)
			x = ser.read(2)
			DTR = int(x,16)
			ser.read(1)
			if(DTR == index + 1): # missed the response
				ser.write('hA3%02X\n'%index) # reload DTR 
				N_CR = ser.read(2) 
				print(' r '),
			elif(DTR == index) : # missed the query
				print(' q '),
			else :
				print('??'), # DTR really out of synce
			ser.write('h%02xC5\n'%device) # read again
			x = ser.read(1)
			retry += 1
			if(retry == 3 ):
				return 0

		data=ser.read(2)
		ser.read(1)
	else : # normal read
		data=ser.read(2)
		ser.read(1)

	index = index + 1
	seed = seed*256 + int(data,base)
	return seed

def lsbFirst(address,base) :
	lsb = readData(0,address,base)
	return readData(0,address,base)*256 + lsb

def SetBank(Bank, Addr):
	global index
	ser.write('hC3%02x\n'%Bank)
	x=ser.read(2)
	ser.write('hA3%02x\n'%Addr)
	index = Addr
	x=ser.read(2)
	return

def WriteBank(address,data) :
	device = address*2 + 1
	ser.write('t%02x81\n'%(device))
	x=ser.read(4)
	ser.write('hC9%02X\n'%(data))   # set NoCal
	x=ser.read(2)
	print(' write %d'%data)

def SetLevel(address, level) :
	ser.write('h%02X%02X\n'%(address*2,level))   # turn on 50%
	x=ser.read(2)
	test=16
	while( test&16 ) :
		test = SendCmd(address,144)
		if(test&16) :
			time.sleep(1)

def SendCmd(address, cmd) :
	ser.write('h%02X%02X\n'%(address*2+1,cmd))   # turn on 50%
	x=ser.read(1)
	result = -1
	if( x== 'N' ):
		ser.read(1)
		return -1
	if( x== 'J' ):
		x = ser.read(2)
		result = int(x,16)
		ser.read(1)
	else :
		print(' bad %s'%x)
	return result

print('\n      ATX LED Consultants Inc - Set Right=12v mode ')

text = input("Which DALI address ( 0-63, 127=all ) : ")
address = int(text)

SetBank(0,9)

fw = readData(0,address,16)
print('DR2  FW version: %02d'%(fw)),
hwv = readData(0,address,16)
print('HW version: %02d'%(hwv))



SetBank(5,9)
LedMode = readData(0,address,16)

if(LedMode != -1 ):
	value = LedMode
	print(' Address %02d Led Mode is '%(address)),
	if( value == 0 ) :
		print(' Auto ')
	if( value == 1 ) :
		print(' Split ')
	if( value == 2 ) :
		print(' Fan ')
	if( value == 3 ) :
		print(' CCT ')
	if( value == 4 ) :
		print(' LEDx8 ')
	if( value == 5 ) :
		print(' PIR ')
	if( value == 6 ) :
		print(' 12vdc ')
	if( value == 7 ) :
		print(' Bad ')
	if( value == 8 ) :
		print(' NotCCT ')

SetLevel(address,0)

SendCmd(address,32)
SendCmd(address,32)

SetLevel(address,0)

SetBank(0,15)
WriteBank(address,16) # set no calibration, 3-Way

SetBank(5,9)
WriteBank(address,6) # set 12v mode

print(' Set to 12v, 50% ') # this will trigger a fast re-cal
SetLevel(address,128)

loop= 4
while (loop) :
	x = SendCmd(address,144)
	if( x >= 0 ) :
		if( x & 0x30 == 0) :
			loop = loop - 1
			if(loop == 3) :  # in 12v mode
				SetLevel(address,0)  # start off
			if(loop == 2) :  # go to CCT mode
				SetBank(0,15)
#				WriteBank(address,0) # set cal OK
				SetBank(5,9)
				WriteBank(address,0) # set Auto
				SendCmd(address,32)
				SendCmd(address,32)
				SetLevel(address,128)
			if(loop == 1) : # turn off
				SetLevel(address,0)
		binary = format(x,'08b')
		print ('status = %s'%(binary))
		y=ser.read(1)
		time.sleep(1)
		SetBank(5,56)
		Right = lsbFirst(address,16)
		Left = lsbFirst(address,16)
		print(' Right %d mV, Left %d mV'%(Right,Left))



