#!/usr/bin/python
#
# program to manage AL-wS-DR1-PIR functions
#
#

import RPi.GPIO as GPIO
import time
import serial
import sys

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=.2,
)

GPIO.setmode(GPIO.BCM)
GPIO.setup(5,GPIO.IN)
GPIO.setup(6,GPIO.IN)
 

def SetBank(Bank,Location) :
	global index
	ser.write('hC3%02X\n'%Bank)  # set Bank address
	x=ser.read(2)
	ser.write('hA3%02X\n'%Location)
	index = Location
	x=ser.read(2)
	return

def ClearFiFo(what) : 
	x=ser.read(1)
	while(x != "" and x != ''):
		if(what != x):
			print('%s'%x),
		if(x == 'H'):
			d0 = int(ser.read(2),16)
			d1 = int(ser.read(2),16)
			print('%02x %02x [ %2d %2d]'%(d0,d1,d0/2,d1))
			crlf = ser.read(1)
		if(x == 'V'):
			result = ser.read(7)
			print('%s'%result)
		if(x == 'L'):
			result = ser.read(7)
			print('%s'%result)
		if(x == 'M'):
			result = ser.read(9)
			print('%s'%result)
		if(x == 'J'):
			result = ser.read(2)
			crlf = ser.read(1)
			if(what == 'J'):
				return result
			print('%s'%result)
		if(x == 'N'):
			print('Nul')
			crlf = ser.read(1)
		x=ser.read(1)
	return -1

def ReadByte(cmd, device) : 
	ser.write('h%02X%02X\n'%(device,cmd))
	Result = ClearFiFo("J")
	time.sleep(.1)
	return int(Result,16)

def NoFade(device) : 
	FadeReady=16  # read a device after fade is done
	while(FadeReady != 0):
		FadeReady = ReadByte(144,device) & 0x10
		time.sleep(.5)
	time.sleep(1)
	return

def readData(seed,device,base) :
	ser.write('h%02xC5\n'%device)  # read a byte from a device
	Result = ClearFiFo('J')
	if(Result == -1) :
		return seed
	seed = seed*256 + int(Result,base)
	return seed

def lsbFirst(device,base) :
	lsb = readData(0,device,base) # Read 2 bytes, LSB first
	return readData(0,device,base)*256 + lsb

def WriteBank(device,data) :
	ser.write('t%02x81\n'%device) # enable
	twice=ser.read(4)
	ser.write('hC7%02X\n'%data) # 
	crlf=ser.read(4)
	time.sleep(0.6) # write to memory
	print('      write %d'%data)
	return

ser.read(10000)

print('\n      ATX LED Consultants Inc - Read info for Topology \n')

ser.write('v\n')  # lets get version number first
x=ser.read()
if x == "" :
	print('  Serial Port Conflict  \n\n')
	exit()

x=ser.read(2)
print('Hat HW version: %s'%(x))
y=ser.read(2)
y=int(y,10)
print('Hat FW version: %d'%(y))
x=ser.read(2)
if x != "" :
	print('Hat Type %s'%(x))
ser.read(1)

if len(sys.argv) < 2 :
	text = input("input address to read : ")
else:
	text = sys.argv[1]

device = int(text)*2+1

# dtr1, 0 as 0:3 

ser.read(1000)

SetBank(0,9)
fw = readData(0,device,16)
if(fw == 0) : 	
	print(' no device ')
	exit()
print('FW version: %02d'%(fw))
hwv = readData(0,device,16)
print('HW version: %02d'%(hwv))
hwt = ReadByte(166,device)
print('HW type: %02d'%(hwv))

SetBank(0,25)
ser.write('t%02x81\n'%device) # enable
twice=ser.read(4)
ser.write('hC701\n') # debug on
data=ser.read(4)


while (( hwt == 1 and hwv > 3) or (hwt == 9)) :
	ClearFiFo("1")
	SetBank(0,16)
	print('16 On Time          %4d seconds'% (readData(0,device,16)*30))
	print('17 Keep On Motion   %4d'% (readData(0,device,16)*16))
	print('18 Turn On Motion   %4d'% (readData(0,device,16)*16))
	print('19 NL Brightness    %4d'% (readData(0,device,16)))
	print('20 Daylight Harvest %4d'% (readData(0,device,16)))
	print('21 Fade Down Time   %4d seconds'% (readData(0,device,16)))
	print('22 NL Threshold     %4d'% (readData(0,device,16)))
	print('23 NL Mode          %4d'% (readData(0,device,16)))
	mode = readData(0,device,16)
	if( mode == 0):
		print('24 Occupancy %d'%mode)
	if( mode == 1):
		print('24 Vacancy %d'%mode)
	if( mode == 2):
		print('24 Manual %d'%mode)
	if( mode == 3):
		print('24 Occupancy Harvest %d'%mode)
	if( mode == 4):
		print('24 Vacancy Harvest %d'%mode)
	if( mode == 5):
		print('24 Manual Harvest %d'%mode)

	SetBank(4,20)
	print('   Ambient Now    %4d'% (readData(0,device,16)))
	print('   Ambient Dark   %4d'% (readData(0,device,16)))
	print('   Last Motion    %4d'% (readData(0,device,16)*16))

	Register = raw_input("Function / Register (a, r, s, l, o, q) ")

	ClearFiFo("1")
	if(device < 32):
		gdevice = device + 0x80;
	else:
		gdevice = device;

	if(Register == 's'): # set Motion in automatic mode
		SetBank(0,25)
		WriteBank(device,3) # auto
		continue
	if(Register == 'a'): # enable Ambient logging
		SetBank(0,25)
		WriteBank(device,5) # ambient logging
		continue
	if(Register == 'q'): # quit
		SetBank(0,25)
		WriteBank(device,0) # debug off
		exit()
	if(Register == 'l'):
		print('Turn Light ON')
		ser.write('h%02x80\n'%(gdevice-1)) # set to 50%
		continue
	if(Register == 'o'):
		print('Turn Light OFF')
		ser.write('h%02x00\n'%gdevice) # off
		continue
	if(Register == 'r'):
		SetBank(0,25)
		WriteBank(device,1) # force debug
		continue
	Register = int(Register,10)
	Value = raw_input("Value ")
	ser.read(10000)
	if(Value == 'q'):
		exit()
	Value = int(Value,10)
	if(Register == 16):
		Value = (Value+29) /30
	if(Register == 17 or Register == 18):
		Value = Value /16

	SetBank(0,Register)
	WriteBank(device,Value) # data

