#
#
# program to set RGBW level
#

import time
import serial
import sys
import math
from decimal import Decimal

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=1,
)


def ReadCmd(Address2, cmd):

	ser.write('h%02X%02x\n'%(Address2+1,cmd)) # get register value
	x=ser.read(1) # j
	if(x == 'J') :
		Return=ser.read(2)
	else:
		Return= 'none'
	crlf=ser.read(1)
	return Return


def ReadCmdDT8(Address2, cmd):

	ser.write('hC108\n')   # enable DT8
	x=ser.read(2)

	ser.write('h%02X%02x\n'%(Address2+1,cmd)) # get 209 value
	x=ser.read(1) # j
	if(x == 'J') :
		Return=ser.read(2)
	else:
		Return= 'none'
	crlf=ser.read(1)
	return Return


def ReadCmdDT8RGB(Address2, cmd, channel):

	SetDTR(channel) # DTR
	
	ser.write('hC108\n')   # enable DT8
	x=ser.read(2)

	ser.write('h%02X%02x\n'%(Address2+1,cmd)) # get RGB value for that channel
	x=ser.read(1) # j
	if(x == 'J') :
		Return=ser.read(2)
	else:
		Return= 'none'
	crlf=ser.read(1)
	return Return

def SetDTR(value):
	ser.write('hA3%02X\n'%value) #DTR
	ser.read(2)

def SetDTR1(value):
	ser.write('hC3%02X\n'%value) #DTR1
	ser.read(2)

def SetDTR2(value):
	ser.write('hC5%02X\n'%value) #DTR2
	ser.read(2)


print('\n      ATX LED Consultants Inc - Detect and set RGB mode ')

text = input("Which DALI address ( 0-63, 127=all ) : ")
address = int(text)*2 

x=ser.read(1000)
DType = ReadCmd(address,0x99)
DVer =  ReadCmdDT8(address,0x97)
DV1  =  ReadCmdDT8(address,0xF7) # 247 features
print(' Type %s Version %s Features %s'%(DType,DVer,DV1))

CStatus =  ReadCmdDT8(address,0xF8) # 248 status
Ctype  =  ReadCmdDT8(address,0xF9) # 249 color type
Ext    =  ReadCmdDT8(address,0xFF) # 255 ext version
print(' Status %s Color Type %s Ext Version %d'%(CStatus, Ctype, int(Ext,16) ))

if( int(DType,16) == 8 and int(Ext,16) == 209 ) : # old old DR2 and 010v
	CStatus = "20"  

if( CStatus == 'none' ) :
	print(' Address %02d not DT8 '%(address/2))
	sys.exit()

Type = int(CStatus,16)

if( Type & 0x20 ) :
	print(' Address %02d is CCT '%(address/2)),
	ReadCmdDT8RGB(address,250,226) # 255 color value
	CCTmsb =  ReadCmd(address,156) #DTR1
	CCTlsb =  ReadCmd(address,152) #DTR

	print(' %s %s '%(CCTlsb, CCTmsb) )

	print('Color = %d'%( 5000 - (( int(CCTlsb,16) + int(CCTmsb,16)*256)/3.5)* 23 ) )
	if( Type & 0xD0 == 0 ):
		sys.exit()    # only CCT, no RGB too

text = "F"

# Ctype values
#           - 0000 0010 is CCT
#           - 001X XXXX is RGB MiBoxer
#           - 011X XXXX is RGB DT8
#           - 100X XXXX is RGBW
#           - 101X XXXX is RGBWA
#	    - 110X XXXX is RGBWAF

CType = int(Ctype,16) & 0XE0 

while ( text != 'q') :

	if( CType >= 0x10 ) :
		Red    =  ReadCmdDT8RGB(address,250,9) # red
		Green  =  ReadCmdDT8RGB(address,250,10) # green
		Blue   =  ReadCmdDT8RGB(address,250,11) # blue
		ChannelCount = 3
		print(' R= %s G=%s B=%s'%(Red,Green,Blue)),

	if( CType == 0x80 ) :
		White  =  ReadCmdDT8RGB(address,250,12) # cool white channel  W
		print('W=%s'%(White)),
		ChannelCount = 4

	if( CType == 0xA0 ) :
		Amber  =  ReadCmdDT8RGB(address,250,13) # warm white channel  A
		print('A=%s'%(Amber)),
		ChannelCount = 5

	if( CType == 0xC0 ) :
		FreeC  =  ReadCmdDT8RGB(address,250,14) # Free Color channel  F
		print('F=%s'%(FreeC)),
		ChannelCount = 6

	print('\n%d channels'%ChannelCount)

	text = raw_input("\n\n%d channels, Which RGBW value - 0x12345678 ? (hex), q : ")

	if (text == 'r'):
		continue

	if (text == 'q'):
		exit()

	if (text != "") :
		RGBW = int(text,16)

	print '%d'%RGBW

	text = raw_input("Which Level? (dec) : ")

	R = RGBW >> 24
	G = (RGBW >> 16 ) & 255
	B = (RGBW >> 8 ) & 255
	W = (RGBW ) & 255

	if( text != "-1") :

		Level = int(text,10)

		total = R+G+B+W  # this is the 4 channel case,

		if( total > 253 ) :

			Scale = Decimal(total/254) # percentage of the total
			Scale = Decimal(1/Scale)

			if(Scale > 0):
				Scaleln = -3.33 * math.log10(Scale) 
			else :
				Scaleln = 0;

#			Scale = pow(1.6,Scaleln) # MiBoxer
			Scale = pow(1.9,Scaleln) # Sunricher

			R = R/Scale
			G = G/Scale
			B = B/Scale
			W = W/Scale

	else :
		Level = 254

	SetDTR(R) #DTR
	SetDTR1(G) #DTR1
	SetDTR2(B) #DTR2

	ser.write('hC108\n')   # enable DT8
	x=ser.read(2)

	ReadCmdDT8(address,235)

	SetDTR(W) #DTR
	SetDTR1(0) #DTR1
	SetDTR2(0) #DTR2

	ReadCmdDT8(address,236)

	SetDTR(128) #DTR - not sure why 128...

	ser.write('hA380\n')  # DTR = 128
	ser.read(2)
	ser.write('hC108\n')   # enable DT8
	x=ser.read(2)
	ser.write('h%02XED\n'%(address+1)) # 237 save as color
	ser.read(2)
	ser.write('hC108\n')   # enable DT8
	ser.read(2)
	ser.write('t%02XE2\n'%(address+1)) # 226 activate color
	ser.read(4)

	ser.write('h%02X%02X\n'%(address,Level)) # arc
	x=ser.read(2)
