#!/usr/bin/python
#
# program to assign DALI short addresses to factory reset devices
#

import time
import serial
import sys

ser = serial.Serial(
	port ='/dev/ttyS0',
	baudrate = 19200,
	parity=serial.PARITY_NONE,
	stopbits=serial.STOPBITS_ONE,
	bytesize=serial.EIGHTBITS,
	timeout=.1,
)

def getXresponse(cmd,comment,target) :
	global Res
	global Value
	
	loop = 0

	ser.write('%s\n'%cmd)
	Res = ser.read(1)
	while( Res == '') :
		ser.write('%s\n'%cmd) # retry, this is a bug in Hat2i - sometimes bus collisions cause no response, nak is missing
		Res = ser.read(1)
		loop = loop + 1
		if(Res != ''):
			print('%06X Retry %d times %s %s'%(target,loop,cmd,comment))


	if( Res == 'X') :
		why = ser.read(1)
#		print('%06X %s collision %s'%(target,comment,why))
		crlf = ser.read(1)
	elif ( Res == 'J') :
		why = ser.read(2)
		Value =int(why,16)/2
#		print('%06X Response %s %d'%(target,comment,Value))
		crlf = ser.read(1)
	elif ( Res == 'N') :
#		print('%06X %s N'%(target,comment))
		crlf = ser.read(1)
	else :
		Res2 = ser.read(100)
		print('%06X %s Other --.%s--.%s--'%(target,comment,Res,Res2))
	return

def SendLong(search):
	global oldhi
	global oldmi
	global oldlo
	straggler = 0
	hi = search >> 16
	mi = (search >> 8 ) & 255
	lo = (search ) & 255

	z = ser.read(100)
	y = z.count('\n')
	if(y > 0):
		print(' straggler %d...'%y)

	readcnt = 0
	if(hi != oldhi):
		ser.write('hB1%02X\n'%hi)  # hi
#		print('w'),
		time.sleep(.05)
		oldhi = hi
		readcnt += 2
	if(mi != oldmi):
		ser.write('hB3%02X\n'%mi)  # mi
#		print('w'),
		time.sleep(.05)
		oldmi = mi
		readcnt += 2
	if(lo != oldlo):
		ser.write('hB5%02X\n'%lo)  # lo
#		print('w'),
		oldlo = lo
		readcnt += 2
	x=ser.read(1)

	while(readcnt > 0):
		if(x == 'X'):
			ser.read(1)
			straggler += 1
			crlf=ser.read(1)
		if(x == 'J'):
			ser.read(2)
			straggler += 1
			print(' %d straggler '%(straggler))
			crlf=ser.read(1)
		if(x == 'H'):
			ser.read(4)
			crlf=ser.read(1)
		if(x == 'N'):
			readcnt -= 2
			crlf=ser.read(1)
		if(x == '' and readcnt > 0 ):
			print(' null %d'%readcnt)
			break
		if(readcnt == 0 ):
			break
		x = ser.read(1)

	if(straggler > 0):
		print(' %d straggler '%(straggler))


UsedSA = [''] * 64

print('\n      ATX LED Consultants Inc - Assign DALI Addresses \n')

ser.write('tA100\n')  # terminate
ser.read(4)


if len(sys.argv) > 1 :
	addr = int((sys.argv[1]),10)*2 + 1
	ser.write('hA3FF\n')
	ser.read(2)
	ser.write('t%02X80\n'%addr) # remove by setting to 255
	ser.read(4)
	ser.write('tA5FF\n')  # start search unassigned
	ser.read(4)
	ser.write('tA700\n')  # randomize
	ser.read(4)

else:
	ser.write('tA500\n')  # start search unassigned
	ser.read(4)
	ser.write('tA700\n')  # randomize
	ser.read(4)

search = 0x00FFFFFF
lastgood = -1
Value = 256
NewSA = 16

done = 0
oldhi = 256
oldmi = 256
oldlo = 256
bit = 24
count=1

while( bit > -1 ) :

	SendLong(search)
	getXresponse("hA900","compare ",search)
	if( Res == 'N'): # too far - no response
		if(bit != 24):
			search = search ^ (1 << bit)
		else:
			break # bit 24 and no response - done
	else: 
		lastgood = search

	bit = bit -1

	if(bit == -1 and lastgood != -1 ):
		SendLong(lastgood)
		getXresponse("hBB00","short is",search)
		if(Res != 'J'):
			bit = 24 # bogus match - retry
			search = 0xFFFFFF
			lastgood = -1
			x=ser.read(100)
			print('   Retry %s'%x)
			continue;
		if(Value < 64):
			if(UsedSA[Value] == 2):
				print('   Duplicate %d'%Value)
				Value=127
			else:
				UsedSA[Value] = 2 # mark used

		if(Value == 127 ) : # not assigned
			Res = 'J'
			NewSA = 15
			while (Res != 'N' ):
				NewSA +=1
				if(NewSA == 64):
					NewSA = 0
				if(UsedSA[NewSA] == ''):
					CMD = 'h%02XA0\n'%(NewSA*2+1) # ping address
					getXresponse(CMD,"query ",search)
					UsedSA[NewSA] = 1; # mark used
					Value = NewSA
				if(NewSA == 15):
					break;
			print('set Short Address %d'%NewSA)
			ser.write('tB7%02X\n'%(NewSA*2+1)) # set new
			ser.read(4)

		print('                            %2d Finished %2d %06X'%(count,Value,search))
		count += 1
		ser.write('hAB00\n') # withdraw
		ser.read(2)
		x = ser.read(2)
		bit = 24
		lastgood = -1
		search = 0xFFFFFF
	else:
		search = search ^ (1 << bit)

ser.write('tA100\n')  # terminate
ser.read(4)
